<?php

namespace Bixo\Faq\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Faq\Http\Requests\CategoriesRequest;
use Bixo\Faq\Interfaces\CategoriesRepositoryInterface;
use Bixo\Faq\Models\Categories;
use Bixo\Faq\Forms\Categories as Form;

/**
 * APIController  class for categories.
 */
class CategoriesAPIController extends BaseController
{

    /**
     * Initialize categories resource controller.
     *
     * @param type CategoriesRepositoryInterface $categories
     *
     * @return null
     */
    public function __construct(CategoriesRepositoryInterface $categories)
    {
        parent::__construct();
        $this->repository = $categories;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Faq\Repositories\Criteria\CategoriesResourceCriteria::class);
    }

    /**
     * Display a list of categories.
     *
     * @return Response
     */
    public function index(CategoriesRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Faq\Repositories\Presenter\CategoriesPresenter::class)
            ->paginate();
    }

    /**
     * Display categories.
     *
     * @param Request $request
     * @param Model   $categories
     *
     * @return Response
     */
    public function show(CategoriesRequest $request, Categories $categories)
    {
        return $categories->setPresenter(\Bixo\Faq\Repositories\Presenter\CategoriesListPresenter::class);
        ;
    }

    /**
     * Create new categories.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CategoriesRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('faq::categories.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('faq/categories/' . $categories->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('faq/categories');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the categories.
     *
     * @param Request $request
     * @param Model   $categories
     *
     * @return Response
     */
    public function update(CategoriesRequest $request, Categories $categories)
    {
        try {
            $data = $request->all();

            $categories->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('faq::categories.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('faq/categories/' . $categories->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('faq/categories/' . $categories->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the categories.
     *
     * @param Model   $categories
     *
     * @return Response
     */
    public function destroy(CategoriesRequest $request, Categories $categories)
    {
        try {
            $categories->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('faq::categories.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('faq/categories/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('faq/categories/' . $categories->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
